/* ------------------------------------------------------------
author: "Grame"
copyright: "(c)GRAME 2006"
license: "BSD"
name: "karplus32"
version: "1.0"
Code generated with Faust 2.50.6 (https://faust.grame.fr)
Compilation options: -a puredata.cpp -lang cpp -es 1 -mcd 16 -single -ftz 0
------------------------------------------------------------ */

#ifndef  __mydsp_H__
#define  __mydsp_H__

/************************************************************************
 ************************************************************************
    FAUST Architecture File
    Copyright (C) 2006-2011 Albert Graef <Dr.Graef@t-online.de>
    ---------------------------------------------------------------------
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as 
    published by the Free Software Foundation; either version 2.1 of the 
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA. 
 ************************************************************************
 ************************************************************************/

/* Pd architecture file, written by Albert Graef <Dr.Graef@t-online.de>.
   This was derived from minimal.cpp included in the Faust distribution.
   Please note that this is to be compiled as a shared library, which is
   then loaded dynamically by Pd as an external. */

#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "faust/misc.h"
#include "faust/gui/UI.h"
#include "faust/gui/meta.h"
#include "faust/dsp/dsp.h"

/******************************************************************************
*******************************************************************************

							       VECTOR INTRINSICS

*******************************************************************************
*******************************************************************************/


/***************************************************************************
   Pd UI interface
 ***************************************************************************/

enum ui_elem_type_t {
  UI_BUTTON, UI_CHECK_BUTTON,
  UI_V_SLIDER, UI_H_SLIDER, UI_NUM_ENTRY,
  UI_V_BARGRAPH, UI_H_BARGRAPH,
  UI_END_GROUP, UI_V_GROUP, UI_H_GROUP, UI_T_GROUP
};

struct ui_elem_t {
  ui_elem_type_t type;
  char *label;
  float *zone;
  float init, min, max, step;
};

class PdUI : public UI
{
public:
  const char *name;
  int nelems, level;
  ui_elem_t *elems;
		
  PdUI();
  PdUI(const char *nm, const char *s);
  virtual ~PdUI();

protected:
  std::string path;
  void add_elem(ui_elem_type_t type, const char *label = NULL);
  void add_elem(ui_elem_type_t type, const char *label, float *zone);
  void add_elem(ui_elem_type_t type, const char *label, float *zone,
		float init, float min, float max, float step);
  void add_elem(ui_elem_type_t type, const char *label, float *zone,
		float min, float max);

public:
  virtual void addButton(const char* label, float* zone);
  virtual void addCheckButton(const char* label, float* zone);
  virtual void addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step);
  virtual void addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step);
  virtual void addNumEntry(const char* label, float* zone, float init, float min, float max, float step);

  virtual void addHorizontalBargraph(const char* label, float* zone, float min, float max);
  virtual void addVerticalBargraph(const char* label, float* zone, float min, float max);
    
  virtual void addSoundfile(const char* label, const char* filename, Soundfile** sf_zone) {}
  
  virtual void openTabBox(const char* label);
  virtual void openHorizontalBox(const char* label);
  virtual void openVerticalBox(const char* label);
  virtual void closeBox();
	
  virtual void run();
};

static std::string mangle(const char *name, int level, const char *s)
{
  const char *s0 = s;
  std::string t = "";
  if (!s) return t;
  // Get rid of bogus "0x00" labels in recent Faust revisions. Also, for
  // backward compatibility with old Faust versions, make sure that default
  // toplevel groups and explicit toplevel groups with an empty label are
  // treated alike (these both return "0x00" labels in the latest Faust, but
  // would be treated inconsistently in earlier versions).
  if (!*s || strcmp(s, "0x00") == 0) {
    if (level == 0)
      // toplevel group with empty label, map to dsp name
      s = name;
    else
      // empty label
      s = "";
  }
  while (*s)
    if (isalnum(*s))
      t += *(s++);
    else {
      const char *s1 = s;
      while (*s && !isalnum(*s)) ++s;
      if (s1 != s0 && *s) t += "-";
    }
  return t;
}

static std::string normpath(std::string path)
{
  path = std::string("/")+path;
  int pos = path.find("//");
  while (pos >= 0) {
    path.erase(pos, 1);
    pos = path.find("//");
  }
  size_t len = path.length();
  if (len > 1 && path[len-1] == '/')
    path.erase(len-1, 1);
  return path;
}

static std::string pathcat(std::string path, std::string label)
{
  if (path.empty())
    return normpath(label);
  else if (label.empty())
    return normpath(path);
  else
    return normpath(path+"/"+label);
}

PdUI::PdUI()
{
  nelems = level = 0;
  elems = NULL;
  name = "";
  path = "";
}

PdUI::PdUI(const char *nm, const char *s)
{
  nelems = level = 0;
  elems = NULL;
  name = nm?nm:"";
  path = s?s:"";
}

PdUI::~PdUI()
{
  if (elems) {
    for (int i = 0; i < nelems; i++)
      if (elems[i].label)
	free(elems[i].label);
    free(elems);
  }
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = NULL;
  elems[nelems].init = 0.0;
  elems[nelems].min = 0.0;
  elems[nelems].max = 0.0;
  elems[nelems].step = 0.0;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = 0.0;
  elems[nelems].min = 0.0;
  elems[nelems].max = 1.0;
  elems[nelems].step = 1.0;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone,
			  float init, float min, float max, float step)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = init;
  elems[nelems].min = min;
  elems[nelems].max = max;
  elems[nelems].step = step;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone,
			  float min, float max)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = 0.0;
  elems[nelems].min = min;
  elems[nelems].max = max;
  elems[nelems].step = 0.0;
  nelems++;
}

void PdUI::addButton(const char* label, float* zone)
{ add_elem(UI_BUTTON, label, zone); }
void PdUI::addCheckButton(const char* label, float* zone)
{ add_elem(UI_CHECK_BUTTON, label, zone); }
void PdUI::addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_V_SLIDER, label, zone, init, min, max, step); }
void PdUI::addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_H_SLIDER, label, zone, init, min, max, step); }
void PdUI::addNumEntry(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_NUM_ENTRY, label, zone, init, min, max, step); }

void PdUI::addHorizontalBargraph(const char* label, float* zone, float min, float max)
{ add_elem(UI_H_BARGRAPH, label, zone, min, max); }
void PdUI::addVerticalBargraph(const char* label, float* zone, float min, float max)
{ add_elem(UI_V_BARGRAPH, label, zone, min, max); }

void PdUI::openTabBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::openHorizontalBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::openVerticalBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::closeBox()
{
  int pos = path.rfind("/");
  if (pos < 0) pos = 0;
  path.erase(pos);
  level--;
}

void PdUI::run() {}

/******************************************************************************
*******************************************************************************

			    FAUST DSP

*******************************************************************************
*******************************************************************************/

//----------------------------------------------------------------------------
//  FAUST generated signal processor
//----------------------------------------------------------------------------
		
#ifndef FAUSTFLOAT
#define FAUSTFLOAT float
#endif 

#include <algorithm>
#include <cmath>
#include <cstdint>

#ifndef FAUSTCLASS 
#define FAUSTCLASS mydsp
#endif

#ifdef __APPLE__ 
#define exp10f __exp10f
#define exp10 __exp10
#endif

#if defined(_WIN32)
#define RESTRICT __restrict
#else
#define RESTRICT __restrict__
#endif


class mydsp : public dsp {
	
 private:
	
	FAUSTFLOAT fHslider0;
	int iRec1[2];
	FAUSTFLOAT fHslider1;
	FAUSTFLOAT fButton0;
	float fVec0[2];
	float fRec2[2];
	FAUSTFLOAT fHslider2;
	int IOTA0;
	float fVec1[4096];
	FAUSTFLOAT fHslider3;
	FAUSTFLOAT fHslider4;
	float fRec0[3];
	FAUSTFLOAT fHslider5;
	float fVec2[4096];
	float fRec3[3];
	float fVec3[4096];
	float fRec4[3];
	float fVec4[4096];
	float fRec5[3];
	float fVec5[4096];
	float fRec6[3];
	float fVec6[4096];
	float fRec7[3];
	float fVec7[4096];
	float fRec8[3];
	float fVec8[4096];
	float fRec9[3];
	float fVec9[4096];
	float fRec10[3];
	float fVec10[4096];
	float fRec11[3];
	float fVec11[4096];
	float fRec12[3];
	float fVec12[4096];
	float fRec13[3];
	float fVec13[4096];
	float fRec14[3];
	float fVec14[4096];
	float fRec15[3];
	float fVec15[2048];
	float fRec16[3];
	float fVec16[512];
	float fRec17[3];
	FAUSTFLOAT fHslider6;
	float fVec17[4096];
	float fRec18[3];
	float fVec18[4096];
	float fRec19[3];
	float fVec19[4096];
	float fRec20[3];
	float fVec20[4096];
	float fRec21[3];
	float fVec21[4096];
	float fRec22[3];
	float fVec22[4096];
	float fRec23[3];
	float fVec23[4096];
	float fRec24[3];
	float fVec24[4096];
	float fRec25[3];
	float fVec25[4096];
	float fRec26[3];
	float fVec26[4096];
	float fRec27[3];
	float fVec27[4096];
	float fRec28[3];
	float fVec28[4096];
	float fRec29[3];
	float fVec29[4096];
	float fRec30[3];
	float fVec30[4096];
	float fRec31[3];
	float fVec31[2048];
	float fRec32[3];
	float fVec32[1024];
	float fRec33[3];
	int fSampleRate;
	
 public:
	
	void metadata(Meta* m) { 
		m->declare("author", "Grame");
		m->declare("compile_options", "-a puredata.cpp -lang cpp -es 1 -mcd 16 -single -ftz 0");
		m->declare("copyright", "(c)GRAME 2006");
		m->declare("filename", "karplus32.dsp");
		m->declare("license", "BSD");
		m->declare("math.lib/author", "GRAME");
		m->declare("math.lib/copyright", "GRAME");
		m->declare("math.lib/deprecated", "This library is deprecated and is not maintained anymore. It will be removed in August 2017.");
		m->declare("math.lib/license", "LGPL with exception");
		m->declare("math.lib/name", "Math Library");
		m->declare("math.lib/version", "1.0");
		m->declare("music.lib/author", "GRAME");
		m->declare("music.lib/copyright", "GRAME");
		m->declare("music.lib/deprecated", "This library is deprecated and is not maintained anymore. It will be removed in August 2017.");
		m->declare("music.lib/license", "LGPL with exception");
		m->declare("music.lib/name", "Music Library");
		m->declare("music.lib/version", "1.0");
		m->declare("name", "karplus32");
		m->declare("version", "1.0");
	}

	virtual int getNumInputs() {
		return 0;
	}
	virtual int getNumOutputs() {
		return 2;
	}
	
	static void classInit(int sample_rate) {
	}
	
	virtual void instanceConstants(int sample_rate) {
		fSampleRate = sample_rate;
	}
	
	virtual void instanceResetUserInterface() {
		fHslider0 = FAUSTFLOAT(0.1f);
		fHslider1 = FAUSTFLOAT(128.0f);
		fButton0 = FAUSTFLOAT(0.0f);
		fHslider2 = FAUSTFLOAT(0.5f);
		fHslider3 = FAUSTFLOAT(32.0f);
		fHslider4 = FAUSTFLOAT(128.0f);
		fHslider5 = FAUSTFLOAT(1.0f);
		fHslider6 = FAUSTFLOAT(0.5f);
	}
	
	virtual void instanceClear() {
		for (int l0 = 0; l0 < 2; l0 = l0 + 1) {
			iRec1[l0] = 0;
		}
		for (int l1 = 0; l1 < 2; l1 = l1 + 1) {
			fVec0[l1] = 0.0f;
		}
		for (int l2 = 0; l2 < 2; l2 = l2 + 1) {
			fRec2[l2] = 0.0f;
		}
		IOTA0 = 0;
		for (int l3 = 0; l3 < 4096; l3 = l3 + 1) {
			fVec1[l3] = 0.0f;
		}
		for (int l4 = 0; l4 < 3; l4 = l4 + 1) {
			fRec0[l4] = 0.0f;
		}
		for (int l5 = 0; l5 < 4096; l5 = l5 + 1) {
			fVec2[l5] = 0.0f;
		}
		for (int l6 = 0; l6 < 3; l6 = l6 + 1) {
			fRec3[l6] = 0.0f;
		}
		for (int l7 = 0; l7 < 4096; l7 = l7 + 1) {
			fVec3[l7] = 0.0f;
		}
		for (int l8 = 0; l8 < 3; l8 = l8 + 1) {
			fRec4[l8] = 0.0f;
		}
		for (int l9 = 0; l9 < 4096; l9 = l9 + 1) {
			fVec4[l9] = 0.0f;
		}
		for (int l10 = 0; l10 < 3; l10 = l10 + 1) {
			fRec5[l10] = 0.0f;
		}
		for (int l11 = 0; l11 < 4096; l11 = l11 + 1) {
			fVec5[l11] = 0.0f;
		}
		for (int l12 = 0; l12 < 3; l12 = l12 + 1) {
			fRec6[l12] = 0.0f;
		}
		for (int l13 = 0; l13 < 4096; l13 = l13 + 1) {
			fVec6[l13] = 0.0f;
		}
		for (int l14 = 0; l14 < 3; l14 = l14 + 1) {
			fRec7[l14] = 0.0f;
		}
		for (int l15 = 0; l15 < 4096; l15 = l15 + 1) {
			fVec7[l15] = 0.0f;
		}
		for (int l16 = 0; l16 < 3; l16 = l16 + 1) {
			fRec8[l16] = 0.0f;
		}
		for (int l17 = 0; l17 < 4096; l17 = l17 + 1) {
			fVec8[l17] = 0.0f;
		}
		for (int l18 = 0; l18 < 3; l18 = l18 + 1) {
			fRec9[l18] = 0.0f;
		}
		for (int l19 = 0; l19 < 4096; l19 = l19 + 1) {
			fVec9[l19] = 0.0f;
		}
		for (int l20 = 0; l20 < 3; l20 = l20 + 1) {
			fRec10[l20] = 0.0f;
		}
		for (int l21 = 0; l21 < 4096; l21 = l21 + 1) {
			fVec10[l21] = 0.0f;
		}
		for (int l22 = 0; l22 < 3; l22 = l22 + 1) {
			fRec11[l22] = 0.0f;
		}
		for (int l23 = 0; l23 < 4096; l23 = l23 + 1) {
			fVec11[l23] = 0.0f;
		}
		for (int l24 = 0; l24 < 3; l24 = l24 + 1) {
			fRec12[l24] = 0.0f;
		}
		for (int l25 = 0; l25 < 4096; l25 = l25 + 1) {
			fVec12[l25] = 0.0f;
		}
		for (int l26 = 0; l26 < 3; l26 = l26 + 1) {
			fRec13[l26] = 0.0f;
		}
		for (int l27 = 0; l27 < 4096; l27 = l27 + 1) {
			fVec13[l27] = 0.0f;
		}
		for (int l28 = 0; l28 < 3; l28 = l28 + 1) {
			fRec14[l28] = 0.0f;
		}
		for (int l29 = 0; l29 < 4096; l29 = l29 + 1) {
			fVec14[l29] = 0.0f;
		}
		for (int l30 = 0; l30 < 3; l30 = l30 + 1) {
			fRec15[l30] = 0.0f;
		}
		for (int l31 = 0; l31 < 2048; l31 = l31 + 1) {
			fVec15[l31] = 0.0f;
		}
		for (int l32 = 0; l32 < 3; l32 = l32 + 1) {
			fRec16[l32] = 0.0f;
		}
		for (int l33 = 0; l33 < 512; l33 = l33 + 1) {
			fVec16[l33] = 0.0f;
		}
		for (int l34 = 0; l34 < 3; l34 = l34 + 1) {
			fRec17[l34] = 0.0f;
		}
		for (int l35 = 0; l35 < 4096; l35 = l35 + 1) {
			fVec17[l35] = 0.0f;
		}
		for (int l36 = 0; l36 < 3; l36 = l36 + 1) {
			fRec18[l36] = 0.0f;
		}
		for (int l37 = 0; l37 < 4096; l37 = l37 + 1) {
			fVec18[l37] = 0.0f;
		}
		for (int l38 = 0; l38 < 3; l38 = l38 + 1) {
			fRec19[l38] = 0.0f;
		}
		for (int l39 = 0; l39 < 4096; l39 = l39 + 1) {
			fVec19[l39] = 0.0f;
		}
		for (int l40 = 0; l40 < 3; l40 = l40 + 1) {
			fRec20[l40] = 0.0f;
		}
		for (int l41 = 0; l41 < 4096; l41 = l41 + 1) {
			fVec20[l41] = 0.0f;
		}
		for (int l42 = 0; l42 < 3; l42 = l42 + 1) {
			fRec21[l42] = 0.0f;
		}
		for (int l43 = 0; l43 < 4096; l43 = l43 + 1) {
			fVec21[l43] = 0.0f;
		}
		for (int l44 = 0; l44 < 3; l44 = l44 + 1) {
			fRec22[l44] = 0.0f;
		}
		for (int l45 = 0; l45 < 4096; l45 = l45 + 1) {
			fVec22[l45] = 0.0f;
		}
		for (int l46 = 0; l46 < 3; l46 = l46 + 1) {
			fRec23[l46] = 0.0f;
		}
		for (int l47 = 0; l47 < 4096; l47 = l47 + 1) {
			fVec23[l47] = 0.0f;
		}
		for (int l48 = 0; l48 < 3; l48 = l48 + 1) {
			fRec24[l48] = 0.0f;
		}
		for (int l49 = 0; l49 < 4096; l49 = l49 + 1) {
			fVec24[l49] = 0.0f;
		}
		for (int l50 = 0; l50 < 3; l50 = l50 + 1) {
			fRec25[l50] = 0.0f;
		}
		for (int l51 = 0; l51 < 4096; l51 = l51 + 1) {
			fVec25[l51] = 0.0f;
		}
		for (int l52 = 0; l52 < 3; l52 = l52 + 1) {
			fRec26[l52] = 0.0f;
		}
		for (int l53 = 0; l53 < 4096; l53 = l53 + 1) {
			fVec26[l53] = 0.0f;
		}
		for (int l54 = 0; l54 < 3; l54 = l54 + 1) {
			fRec27[l54] = 0.0f;
		}
		for (int l55 = 0; l55 < 4096; l55 = l55 + 1) {
			fVec27[l55] = 0.0f;
		}
		for (int l56 = 0; l56 < 3; l56 = l56 + 1) {
			fRec28[l56] = 0.0f;
		}
		for (int l57 = 0; l57 < 4096; l57 = l57 + 1) {
			fVec28[l57] = 0.0f;
		}
		for (int l58 = 0; l58 < 3; l58 = l58 + 1) {
			fRec29[l58] = 0.0f;
		}
		for (int l59 = 0; l59 < 4096; l59 = l59 + 1) {
			fVec29[l59] = 0.0f;
		}
		for (int l60 = 0; l60 < 3; l60 = l60 + 1) {
			fRec30[l60] = 0.0f;
		}
		for (int l61 = 0; l61 < 4096; l61 = l61 + 1) {
			fVec30[l61] = 0.0f;
		}
		for (int l62 = 0; l62 < 3; l62 = l62 + 1) {
			fRec31[l62] = 0.0f;
		}
		for (int l63 = 0; l63 < 2048; l63 = l63 + 1) {
			fVec31[l63] = 0.0f;
		}
		for (int l64 = 0; l64 < 3; l64 = l64 + 1) {
			fRec32[l64] = 0.0f;
		}
		for (int l65 = 0; l65 < 1024; l65 = l65 + 1) {
			fVec32[l65] = 0.0f;
		}
		for (int l66 = 0; l66 < 3; l66 = l66 + 1) {
			fRec33[l66] = 0.0f;
		}
	}
	
	virtual void init(int sample_rate) {
		classInit(sample_rate);
		instanceInit(sample_rate);
	}
	virtual void instanceInit(int sample_rate) {
		instanceConstants(sample_rate);
		instanceResetUserInterface();
		instanceClear();
	}
	
	virtual mydsp* clone() {
		return new mydsp();
	}
	
	virtual int getSampleRate() {
		return fSampleRate;
	}
	
	virtual void buildUserInterface(UI* ui_interface) {
		ui_interface->openVerticalBox("karplus32");
		ui_interface->openVerticalBox("excitator");
		ui_interface->addHorizontalSlider("excitation (samples)", &fHslider1, FAUSTFLOAT(128.0f), FAUSTFLOAT(2.0f), FAUSTFLOAT(512.0f), FAUSTFLOAT(1.0f));
		ui_interface->addButton("play", &fButton0);
		ui_interface->closeBox();
		ui_interface->openVerticalBox("noise generator");
		ui_interface->addHorizontalSlider("level", &fHslider2, FAUSTFLOAT(0.5f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->addHorizontalSlider("output volume", &fHslider6, FAUSTFLOAT(0.5f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(0.1f));
		ui_interface->openVerticalBox("resonator x32");
		ui_interface->addHorizontalSlider("attenuation", &fHslider0, FAUSTFLOAT(0.1f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(0.01f));
		ui_interface->addHorizontalSlider("detune", &fHslider3, FAUSTFLOAT(32.0f), FAUSTFLOAT(0.0f), FAUSTFLOAT(512.0f), FAUSTFLOAT(1.0f));
		ui_interface->addHorizontalSlider("duration (samples)", &fHslider4, FAUSTFLOAT(128.0f), FAUSTFLOAT(2.0f), FAUSTFLOAT(512.0f), FAUSTFLOAT(1.0f));
		ui_interface->addHorizontalSlider("polyphony", &fHslider5, FAUSTFLOAT(1.0f), FAUSTFLOAT(0.0f), FAUSTFLOAT(32.0f), FAUSTFLOAT(1.0f));
		ui_interface->closeBox();
		ui_interface->closeBox();
	}
	
	virtual void compute(int count, FAUSTFLOAT** RESTRICT inputs, FAUSTFLOAT** RESTRICT outputs) {
		FAUSTFLOAT* output0 = outputs[0];
		FAUSTFLOAT* output1 = outputs[1];
		float fSlow0 = 0.5f * (1.0f - float(fHslider0));
		float fSlow1 = 1.0f / float(fHslider1);
		float fSlow2 = float(fButton0);
		float fSlow3 = 4.656613e-10f * float(fHslider2);
		float fSlow4 = float(fHslider3);
		float fSlow5 = float(fHslider4);
		int iSlow6 = int(fSlow5 + 3e+01f * fSlow4 + -1.5f) & 4095;
		float fSlow7 = float(fHslider5);
		float fSlow8 = float(fSlow7 > 3e+01f);
		int iSlow9 = int(fSlow5 + 28.0f * fSlow4 + -1.5f) & 4095;
		float fSlow10 = float(fSlow7 > 28.0f);
		int iSlow11 = int(fSlow5 + 26.0f * fSlow4 + -1.5f) & 4095;
		float fSlow12 = float(fSlow7 > 26.0f);
		int iSlow13 = int(fSlow5 + 24.0f * fSlow4 + -1.5f) & 4095;
		float fSlow14 = float(fSlow7 > 24.0f);
		int iSlow15 = int(fSlow5 + 22.0f * fSlow4 + -1.5f) & 4095;
		float fSlow16 = float(fSlow7 > 22.0f);
		int iSlow17 = int(fSlow5 + 2e+01f * fSlow4 + -1.5f) & 4095;
		float fSlow18 = float(fSlow7 > 2e+01f);
		int iSlow19 = int(fSlow5 + 18.0f * fSlow4 + -1.5f) & 4095;
		float fSlow20 = float(fSlow7 > 18.0f);
		int iSlow21 = int(fSlow5 + 16.0f * fSlow4 + -1.5f) & 4095;
		float fSlow22 = float(fSlow7 > 16.0f);
		int iSlow23 = int(fSlow5 + 14.0f * fSlow4 + -1.5f) & 4095;
		float fSlow24 = float(fSlow7 > 14.0f);
		int iSlow25 = int(fSlow5 + 12.0f * fSlow4 + -1.5f) & 4095;
		float fSlow26 = float(fSlow7 > 12.0f);
		int iSlow27 = int(fSlow5 + 1e+01f * fSlow4 + -1.5f) & 4095;
		float fSlow28 = float(fSlow7 > 1e+01f);
		int iSlow29 = int(fSlow5 + 8.0f * fSlow4 + -1.5f) & 4095;
		float fSlow30 = float(fSlow7 > 8.0f);
		int iSlow31 = int(fSlow5 + 6.0f * fSlow4 + -1.5f) & 4095;
		float fSlow32 = float(fSlow7 > 6.0f);
		int iSlow33 = int(fSlow5 + 4.0f * fSlow4 + -1.5f) & 4095;
		float fSlow34 = float(fSlow7 > 4.0f);
		int iSlow35 = int(fSlow5 + 2.0f * fSlow4 + -1.5f) & 4095;
		float fSlow36 = float(fSlow7 > 2.0f);
		int iSlow37 = int(fSlow5 + -1.5f) & 4095;
		float fSlow38 = float(fSlow7 > 0.0f);
		float fSlow39 = float(fHslider6);
		int iSlow40 = int(fSlow5 + 31.0f * fSlow4 + -1.5f) & 4095;
		float fSlow41 = float(fSlow7 > 31.0f);
		int iSlow42 = int(fSlow5 + 29.0f * fSlow4 + -1.5f) & 4095;
		float fSlow43 = float(fSlow7 > 29.0f);
		int iSlow44 = int(fSlow5 + 27.0f * fSlow4 + -1.5f) & 4095;
		float fSlow45 = float(fSlow7 > 27.0f);
		int iSlow46 = int(fSlow5 + 25.0f * fSlow4 + -1.5f) & 4095;
		float fSlow47 = float(fSlow7 > 25.0f);
		int iSlow48 = int(fSlow5 + 23.0f * fSlow4 + -1.5f) & 4095;
		float fSlow49 = float(fSlow7 > 23.0f);
		int iSlow50 = int(fSlow5 + 21.0f * fSlow4 + -1.5f) & 4095;
		float fSlow51 = float(fSlow7 > 21.0f);
		int iSlow52 = int(fSlow5 + 19.0f * fSlow4 + -1.5f) & 4095;
		float fSlow53 = float(fSlow7 > 19.0f);
		int iSlow54 = int(fSlow5 + 17.0f * fSlow4 + -1.5f) & 4095;
		float fSlow55 = float(fSlow7 > 17.0f);
		int iSlow56 = int(fSlow5 + 15.0f * fSlow4 + -1.5f) & 4095;
		float fSlow57 = float(fSlow7 > 15.0f);
		int iSlow58 = int(fSlow5 + 13.0f * fSlow4 + -1.5f) & 4095;
		float fSlow59 = float(fSlow7 > 13.0f);
		int iSlow60 = int(fSlow5 + 11.0f * fSlow4 + -1.5f) & 4095;
		float fSlow61 = float(fSlow7 > 11.0f);
		int iSlow62 = int(fSlow5 + 9.0f * fSlow4 + -1.5f) & 4095;
		float fSlow63 = float(fSlow7 > 9.0f);
		int iSlow64 = int(fSlow5 + 7.0f * fSlow4 + -1.5f) & 4095;
		float fSlow65 = float(fSlow7 > 7.0f);
		int iSlow66 = int(fSlow5 + 5.0f * fSlow4 + -1.5f) & 4095;
		float fSlow67 = float(fSlow7 > 5.0f);
		int iSlow68 = int(fSlow5 + 3.0f * fSlow4 + -1.5f) & 4095;
		float fSlow69 = float(fSlow7 > 3.0f);
		int iSlow70 = int(fSlow4 + fSlow5 + -1.5f) & 4095;
		float fSlow71 = float(fSlow7 > 1.0f);
		for (int i0 = 0; i0 < count; i0 = i0 + 1) {
			iRec1[0] = 1103515245 * iRec1[1] + 12345;
			fVec0[0] = fSlow2;
			fRec2[0] = fRec2[1] + float((fSlow2 - fVec0[1]) > 0.0f) - fSlow1 * float(fRec2[1] > 0.0f);
			float fTemp0 = fSlow3 * (float(fRec2[0] > 0.0f) + 1.5258789e-05f) * float(iRec1[0]);
			fVec1[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec0[1] + fRec0[2]);
			fRec0[0] = fVec1[(IOTA0 - iSlow6) & 4095];
			fVec2[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec3[1] + fRec3[2]);
			fRec3[0] = fVec2[(IOTA0 - iSlow9) & 4095];
			fVec3[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec4[1] + fRec4[2]);
			fRec4[0] = fVec3[(IOTA0 - iSlow11) & 4095];
			fVec4[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec5[1] + fRec5[2]);
			fRec5[0] = fVec4[(IOTA0 - iSlow13) & 4095];
			fVec5[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec6[1] + fRec6[2]);
			fRec6[0] = fVec5[(IOTA0 - iSlow15) & 4095];
			fVec6[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec7[1] + fRec7[2]);
			fRec7[0] = fVec6[(IOTA0 - iSlow17) & 4095];
			fVec7[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec8[1] + fRec8[2]);
			fRec8[0] = fVec7[(IOTA0 - iSlow19) & 4095];
			fVec8[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec9[1] + fRec9[2]);
			fRec9[0] = fVec8[(IOTA0 - iSlow21) & 4095];
			fVec9[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec10[1] + fRec10[2]);
			fRec10[0] = fVec9[(IOTA0 - iSlow23) & 4095];
			fVec10[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec11[1] + fRec11[2]);
			fRec11[0] = fVec10[(IOTA0 - iSlow25) & 4095];
			fVec11[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec12[1] + fRec12[2]);
			fRec12[0] = fVec11[(IOTA0 - iSlow27) & 4095];
			fVec12[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec13[1] + fRec13[2]);
			fRec13[0] = fVec12[(IOTA0 - iSlow29) & 4095];
			fVec13[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec14[1] + fRec14[2]);
			fRec14[0] = fVec13[(IOTA0 - iSlow31) & 4095];
			fVec14[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec15[1] + fRec15[2]);
			fRec15[0] = fVec14[(IOTA0 - iSlow33) & 4095];
			fVec15[IOTA0 & 2047] = fTemp0 + fSlow0 * (fRec16[1] + fRec16[2]);
			fRec16[0] = fVec15[(IOTA0 - iSlow35) & 2047];
			fVec16[IOTA0 & 511] = fSlow0 * (fRec17[1] + fRec17[2]) + fTemp0;
			fRec17[0] = fVec16[(IOTA0 - iSlow37) & 511];
			output0[i0] = FAUSTFLOAT(fSlow39 * (fSlow38 * fRec17[0] + fSlow36 * fRec16[0] + fSlow34 * fRec15[0] + fSlow32 * fRec14[0] + fSlow30 * fRec13[0] + fSlow28 * fRec12[0] + fSlow26 * fRec11[0] + fSlow24 * fRec10[0] + fSlow22 * fRec9[0] + fSlow20 * fRec8[0] + fSlow18 * fRec7[0] + fSlow16 * fRec6[0] + fSlow14 * fRec5[0] + fSlow12 * fRec4[0] + fSlow10 * fRec3[0] + fSlow8 * fRec0[0]));
			fVec17[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec18[1] + fRec18[2]);
			fRec18[0] = fVec17[(IOTA0 - iSlow40) & 4095];
			fVec18[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec19[1] + fRec19[2]);
			fRec19[0] = fVec18[(IOTA0 - iSlow42) & 4095];
			fVec19[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec20[1] + fRec20[2]);
			fRec20[0] = fVec19[(IOTA0 - iSlow44) & 4095];
			fVec20[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec21[1] + fRec21[2]);
			fRec21[0] = fVec20[(IOTA0 - iSlow46) & 4095];
			fVec21[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec22[1] + fRec22[2]);
			fRec22[0] = fVec21[(IOTA0 - iSlow48) & 4095];
			fVec22[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec23[1] + fRec23[2]);
			fRec23[0] = fVec22[(IOTA0 - iSlow50) & 4095];
			fVec23[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec24[1] + fRec24[2]);
			fRec24[0] = fVec23[(IOTA0 - iSlow52) & 4095];
			fVec24[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec25[1] + fRec25[2]);
			fRec25[0] = fVec24[(IOTA0 - iSlow54) & 4095];
			fVec25[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec26[1] + fRec26[2]);
			fRec26[0] = fVec25[(IOTA0 - iSlow56) & 4095];
			fVec26[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec27[1] + fRec27[2]);
			fRec27[0] = fVec26[(IOTA0 - iSlow58) & 4095];
			fVec27[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec28[1] + fRec28[2]);
			fRec28[0] = fVec27[(IOTA0 - iSlow60) & 4095];
			fVec28[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec29[1] + fRec29[2]);
			fRec29[0] = fVec28[(IOTA0 - iSlow62) & 4095];
			fVec29[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec30[1] + fRec30[2]);
			fRec30[0] = fVec29[(IOTA0 - iSlow64) & 4095];
			fVec30[IOTA0 & 4095] = fTemp0 + fSlow0 * (fRec31[1] + fRec31[2]);
			fRec31[0] = fVec30[(IOTA0 - iSlow66) & 4095];
			fVec31[IOTA0 & 2047] = fTemp0 + fSlow0 * (fRec32[1] + fRec32[2]);
			fRec32[0] = fVec31[(IOTA0 - iSlow68) & 2047];
			fVec32[IOTA0 & 1023] = fTemp0 + fSlow0 * (fRec33[1] + fRec33[2]);
			fRec33[0] = fVec32[(IOTA0 - iSlow70) & 1023];
			output1[i0] = FAUSTFLOAT(fSlow39 * (fSlow71 * fRec33[0] + fSlow69 * fRec32[0] + fSlow67 * fRec31[0] + fSlow65 * fRec30[0] + fSlow63 * fRec29[0] + fSlow61 * fRec28[0] + fSlow59 * fRec27[0] + fSlow57 * fRec26[0] + fSlow55 * fRec25[0] + fSlow53 * fRec24[0] + fSlow51 * fRec23[0] + fSlow49 * fRec22[0] + fSlow47 * fRec21[0] + fSlow45 * fRec20[0] + fSlow43 * fRec19[0] + fSlow41 * fRec18[0]));
			iRec1[1] = iRec1[0];
			fVec0[1] = fVec0[0];
			fRec2[1] = fRec2[0];
			IOTA0 = IOTA0 + 1;
			fRec0[2] = fRec0[1];
			fRec0[1] = fRec0[0];
			fRec3[2] = fRec3[1];
			fRec3[1] = fRec3[0];
			fRec4[2] = fRec4[1];
			fRec4[1] = fRec4[0];
			fRec5[2] = fRec5[1];
			fRec5[1] = fRec5[0];
			fRec6[2] = fRec6[1];
			fRec6[1] = fRec6[0];
			fRec7[2] = fRec7[1];
			fRec7[1] = fRec7[0];
			fRec8[2] = fRec8[1];
			fRec8[1] = fRec8[0];
			fRec9[2] = fRec9[1];
			fRec9[1] = fRec9[0];
			fRec10[2] = fRec10[1];
			fRec10[1] = fRec10[0];
			fRec11[2] = fRec11[1];
			fRec11[1] = fRec11[0];
			fRec12[2] = fRec12[1];
			fRec12[1] = fRec12[0];
			fRec13[2] = fRec13[1];
			fRec13[1] = fRec13[0];
			fRec14[2] = fRec14[1];
			fRec14[1] = fRec14[0];
			fRec15[2] = fRec15[1];
			fRec15[1] = fRec15[0];
			fRec16[2] = fRec16[1];
			fRec16[1] = fRec16[0];
			fRec17[2] = fRec17[1];
			fRec17[1] = fRec17[0];
			fRec18[2] = fRec18[1];
			fRec18[1] = fRec18[0];
			fRec19[2] = fRec19[1];
			fRec19[1] = fRec19[0];
			fRec20[2] = fRec20[1];
			fRec20[1] = fRec20[0];
			fRec21[2] = fRec21[1];
			fRec21[1] = fRec21[0];
			fRec22[2] = fRec22[1];
			fRec22[1] = fRec22[0];
			fRec23[2] = fRec23[1];
			fRec23[1] = fRec23[0];
			fRec24[2] = fRec24[1];
			fRec24[1] = fRec24[0];
			fRec25[2] = fRec25[1];
			fRec25[1] = fRec25[0];
			fRec26[2] = fRec26[1];
			fRec26[1] = fRec26[0];
			fRec27[2] = fRec27[1];
			fRec27[1] = fRec27[0];
			fRec28[2] = fRec28[1];
			fRec28[1] = fRec28[0];
			fRec29[2] = fRec29[1];
			fRec29[1] = fRec29[0];
			fRec30[2] = fRec30[1];
			fRec30[1] = fRec30[0];
			fRec31[2] = fRec31[1];
			fRec31[1] = fRec31[0];
			fRec32[2] = fRec32[1];
			fRec32[1] = fRec32[0];
			fRec33[2] = fRec33[1];
			fRec33[1] = fRec33[0];
		}
	}

};

#include <stdio.h>
#include <string>
#include "m_pd.h"

#define sym(name) xsym(name)
#define xsym(name) #name
#define faust_setup(name) xfaust_setup(name)
#define xfaust_setup(name) name ## _tilde_setup(void)

// time for "active" toggle xfades in secs
#define XFADE_TIME 0.1f

static t_class *faust_class;

struct t_faust {
  t_object x_obj;
#ifdef __MINGW32__
  /* This seems to be necessary as some as yet undetermined Pd routine seems
     to write past the end of x_obj on Windows. */
  int fence; /* dummy field (not used) */
#endif
  mydsp *dsp;
  PdUI *ui;
  std::string *label;
  int active, xfade, n_xfade, rate, n_in, n_out;
  t_sample **inputs, **outputs, **buf;
  t_outlet *out;
  t_sample f;
};

static t_symbol *s_button, *s_checkbox, *s_vslider, *s_hslider, *s_nentry,
  *s_vbargraph, *s_hbargraph;

static inline void zero_samples(int k, int n, t_sample **out)
{
  for (int i = 0; i < k; i++)
#ifdef __STDC_IEC_559__
    /* IEC 559 a.k.a. IEEE 754 floats can be initialized faster like this */
    memset(out[i], 0, n*sizeof(t_sample));
#else
    for (int j = 0; j < n; j++)
      out[i][j] = 0.0f;
#endif
}

static inline void copy_samples(int k, int n, t_sample **out, t_sample **in)
{
  for (int i = 0; i < k; i++)
    memcpy(out[i], in[i], n*sizeof(t_sample));
}

static t_int *faust_perform(t_int *w)
{
  t_faust *x = (t_faust *)(w[1]);
  int n = (int)(w[2]);
  if (!x->dsp || !x->buf) return (w+3);
  AVOIDDENORMALS;
  if (x->xfade > 0) {
    float d = 1.0f/x->n_xfade, f = (x->xfade--)*d;
    d = d/n;
    x->dsp->compute(n, x->inputs, x->buf);
    if (x->active)
      if (x->n_in == x->n_out)
	/* xfade inputs -> buf */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->inputs[i][j]+(1.0f-f)*x->buf[i][j];
      else
	/* xfade 0 -> buf */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = (1.0f-f)*x->buf[i][j];
    else
      if (x->n_in == x->n_out)
	/* xfade buf -> inputs */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->buf[i][j]+(1.0f-f)*x->inputs[i][j];
      else
	/* xfade buf -> 0 */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->buf[i][j];
  } else if (x->active) {
    x->dsp->compute(n, x->inputs, x->buf);
    copy_samples(x->n_out, n, x->outputs, x->buf);
  } else if (x->n_in == x->n_out) {
    copy_samples(x->n_out, n, x->buf, x->inputs);
    copy_samples(x->n_out, n, x->outputs, x->buf);
  } else
    zero_samples(x->n_out, n, x->outputs);
  return (w+3);
}

static void faust_dsp(t_faust *x, t_signal **sp)
{
  int n = sp[0]->s_n, sr = (int)sp[0]->s_sr;
  if (x->rate <= 0) {
    /* default sample rate is whatever Pd tells us */
    PdUI *ui = x->ui;
    float *z = NULL;
    if (ui->nelems > 0 &&
	(z = (float*)malloc(ui->nelems*sizeof(float)))) {
      /* save the current control values */
      for (int i = 0; i < ui->nelems; i++)
	if (ui->elems[i].zone)
	  z[i] = *ui->elems[i].zone;
    }
    /* set the proper sample rate; this requires reinitializing the dsp */
    x->rate = sr;
    x->dsp->init(sr);
    if (z) {
      /* restore previous control values */
      for (int i = 0; i < ui->nelems; i++)
	if (ui->elems[i].zone)
	  *ui->elems[i].zone = z[i];
      free(z);
    }
  }
  if (n > 0)
    x->n_xfade = (int)(x->rate*XFADE_TIME/n);
  dsp_add(faust_perform, 2, x, n);
  for (int i = 0; i < x->n_in; i++)
    x->inputs[i] = sp[i+1]->s_vec;
  for (int i = 0; i < x->n_out; i++)
    x->outputs[i] = sp[x->n_in+i+1]->s_vec;
  if (x->buf != NULL)
    for (int i = 0; i < x->n_out; i++) {
      x->buf[i] = (t_sample*)malloc(n*sizeof(t_sample));
      if (x->buf[i] == NULL) {
	for (int j = 0; j < i; j++)
	  free(x->buf[j]);
	free(x->buf);
	x->buf = NULL;
	break;
      }
    }
}

static int pathcmp(const char *s, const char *t)
{
  int n = strlen(s), m = strlen(t);
  if (n == 0 || m == 0)
    return 0;
  else if (t[0] == '/')
    return strcmp(s, t);
  else if (n <= m || s[n-m-1] != '/')
    return strcmp(s+1, t);
  else
    return strcmp(s+n-m, t);
}

static void faust_any(t_faust *x, t_symbol *s, int argc, t_atom *argv)
{
  if (!x->dsp) return;
  PdUI *ui = x->ui;
  if (s == &s_bang) {
    for (int i = 0; i < ui->nelems; i++)
      if (ui->elems[i].label && ui->elems[i].zone) {
	t_atom args[6];
	t_symbol *_s;
	switch (ui->elems[i].type) {
	case UI_BUTTON:
	  _s = s_button;
	  break;
	case UI_CHECK_BUTTON:
	  _s = s_checkbox;
	  break;
	case UI_V_SLIDER:
	  _s = s_vslider;
	  break;
	case UI_H_SLIDER:
	  _s = s_hslider;
	  break;
	case UI_NUM_ENTRY:
	  _s = s_nentry;
	  break;
	case UI_V_BARGRAPH:
	  _s = s_vbargraph;
	  break;
	case UI_H_BARGRAPH:
	  _s = s_hbargraph;
	  break;
	default:
	  continue;
	}
	SETSYMBOL(&args[0], gensym(ui->elems[i].label));
	SETFLOAT(&args[1], *ui->elems[i].zone);
	SETFLOAT(&args[2], ui->elems[i].init);
	SETFLOAT(&args[3], ui->elems[i].min);
	SETFLOAT(&args[4], ui->elems[i].max);
	SETFLOAT(&args[5], ui->elems[i].step);
	outlet_anything(x->out, _s, 6, args);
      }
  } else {
    const char *label = s->s_name;
    int count = 0;
    for (int i = 0; i < ui->nelems; i++)
      if (ui->elems[i].label &&
	  pathcmp(ui->elems[i].label, label) == 0) {
	if (argc == 0) {
	  if (ui->elems[i].zone) {
	    t_atom arg;
	    SETFLOAT(&arg, *ui->elems[i].zone);
	    outlet_anything(x->out, gensym(ui->elems[i].label), 1, &arg);
	  }
	  ++count;
	} else if (argc == 1 &&
		   (argv[0].a_type == A_FLOAT ||
		    argv[0].a_type == A_DEFFLOAT) &&
		   ui->elems[i].zone) {
	  float f = atom_getfloat(argv);
	  *ui->elems[i].zone = f;
	  ++count;
	} else
	  pd_error(x, "[faust] %s: bad control argument: %s",
		   x->label->c_str(), label);
      }
    if (count == 0 && strcmp(label, "active") == 0) {
      if (argc == 0) {
	t_atom arg;
	SETFLOAT(&arg, (float)x->active);
	outlet_anything(x->out, gensym((char*)"active"), 1, &arg);
      } else if (argc == 1 &&
		 (argv[0].a_type == A_FLOAT ||
		  argv[0].a_type == A_DEFFLOAT)) {
	float f = atom_getfloat(argv);
	x->active = (int)f;
	x->xfade = x->n_xfade;
      }
    }
  }
}

static void faust_free(t_faust *x)
{
  if (x->label) delete x->label;
  if (x->dsp) delete x->dsp;
  if (x->ui) delete x->ui;
  if (x->inputs) free(x->inputs);
  if (x->outputs) free(x->outputs);
  if (x->buf) {
    for (int i = 0; i < x->n_out; i++)
      if (x->buf[i]) free(x->buf[i]);
    free(x->buf);
  }
}

static void *faust_new(t_symbol *s, int argc, t_atom *argv)
{
  t_faust *x = (t_faust*)pd_new(faust_class);
  int sr = -1;
  t_symbol *id = NULL;
  x->active = 1;
  for (int i = 0; i < argc; i++)
    if (argv[i].a_type == A_FLOAT || argv[i].a_type == A_DEFFLOAT)
      sr = (int)argv[i].a_w.w_float;
    else if (argv[i].a_type == A_SYMBOL || argv[i].a_type == A_DEFSYMBOL)
      id = argv[i].a_w.w_symbol;
  x->rate = sr;
  if (sr <= 0) sr = 44100;
  x->xfade = 0; x->n_xfade = (int)(sr*XFADE_TIME/64);
  x->inputs = x->outputs = x->buf = NULL;
    x->label = new std::string(sym(mydsp) "~");
  x->dsp = new mydsp();
  x->ui = new PdUI(sym(mydsp), id?id->s_name:NULL);
  if (!x->dsp || !x->ui || !x->label) goto error;
  if (id) {
    *x->label += " ";
    *x->label += id->s_name;
  }
  x->n_in = x->dsp->getNumInputs();
  x->n_out = x->dsp->getNumOutputs();
  if (x->n_in > 0)
    x->inputs = (t_sample**)malloc(x->n_in*sizeof(t_sample*));
  if (x->n_out > 0) {
    x->outputs = (t_sample**)malloc(x->n_out*sizeof(t_sample*));
    x->buf = (t_sample**)malloc(x->n_out*sizeof(t_sample*));
  }
  if ((x->n_in > 0 && x->inputs == NULL) ||
      (x->n_out > 0 && (x->outputs == NULL || x->buf == NULL)))
    goto error;
  for (int i = 0; i < x->n_out; i++)
    x->buf[i] = NULL;
  x->dsp->init(sr);
  x->dsp->buildUserInterface(x->ui);
  for (int i = 0; i < x->n_in; i++)
    inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
  x->out = outlet_new(&x->x_obj, 0);
  for (int i = 0; i < x->n_out; i++)
    outlet_new(&x->x_obj, &s_signal);
  return (void *)x;
 error:
  faust_free(x);
  x->dsp = NULL; x->ui = NULL;
  x->inputs = x->outputs = x->buf = NULL;
  return (void *)x;
}

extern "C" void faust_setup(mydsp)
{
  t_symbol *s = gensym(sym(mydsp) "~");
  faust_class =
    class_new(s, (t_newmethod)faust_new, (t_method)faust_free,
	      sizeof(t_faust), CLASS_DEFAULT,
	      A_GIMME, A_NULL);
  class_addmethod(faust_class, (t_method)faust_dsp, gensym((char*)"dsp"), A_NULL);
  class_addanything(faust_class, faust_any);
  class_addmethod(faust_class, nullfn, &s_signal, A_NULL);
  s_button = gensym((char*)"button");
  s_checkbox = gensym((char*)"checkbox");
  s_vslider = gensym((char*)"vslider");
  s_hslider = gensym((char*)"hslider");
  s_nentry = gensym((char*)"nentry");
  s_vbargraph = gensym((char*)"vbargraph");
  s_hbargraph = gensym((char*)"hbargraph");
  /* give some indication that we're loaded and ready to go */
  mydsp dsp = mydsp();
  post("[faust] %s: %d inputs, %d outputs", sym(mydsp) "~",
       dsp.getNumInputs(), dsp.getNumOutputs());
}

#endif
