/* ------------------------------------------------------------
author: "Grame"
copyright: "(c)GRAME 2006"
license: "BSD"
name: "multibandfilter"
version: "1.0"
Code generated with Faust 2.50.6 (https://faust.grame.fr)
Compilation options: -a puredata.cpp -lang cpp -es 1 -mcd 16 -single -ftz 0
------------------------------------------------------------ */

#ifndef  __mydsp_H__
#define  __mydsp_H__

/************************************************************************
 ************************************************************************
    FAUST Architecture File
    Copyright (C) 2006-2011 Albert Graef <Dr.Graef@t-online.de>
    ---------------------------------------------------------------------
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as 
    published by the Free Software Foundation; either version 2.1 of the 
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA. 
 ************************************************************************
 ************************************************************************/

/* Pd architecture file, written by Albert Graef <Dr.Graef@t-online.de>.
   This was derived from minimal.cpp included in the Faust distribution.
   Please note that this is to be compiled as a shared library, which is
   then loaded dynamically by Pd as an external. */

#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "faust/misc.h"
#include "faust/gui/UI.h"
#include "faust/gui/meta.h"
#include "faust/dsp/dsp.h"

/******************************************************************************
*******************************************************************************

							       VECTOR INTRINSICS

*******************************************************************************
*******************************************************************************/


/***************************************************************************
   Pd UI interface
 ***************************************************************************/

enum ui_elem_type_t {
  UI_BUTTON, UI_CHECK_BUTTON,
  UI_V_SLIDER, UI_H_SLIDER, UI_NUM_ENTRY,
  UI_V_BARGRAPH, UI_H_BARGRAPH,
  UI_END_GROUP, UI_V_GROUP, UI_H_GROUP, UI_T_GROUP
};

struct ui_elem_t {
  ui_elem_type_t type;
  char *label;
  float *zone;
  float init, min, max, step;
};

class PdUI : public UI
{
public:
  const char *name;
  int nelems, level;
  ui_elem_t *elems;
		
  PdUI();
  PdUI(const char *nm, const char *s);
  virtual ~PdUI();

protected:
  std::string path;
  void add_elem(ui_elem_type_t type, const char *label = NULL);
  void add_elem(ui_elem_type_t type, const char *label, float *zone);
  void add_elem(ui_elem_type_t type, const char *label, float *zone,
		float init, float min, float max, float step);
  void add_elem(ui_elem_type_t type, const char *label, float *zone,
		float min, float max);

public:
  virtual void addButton(const char* label, float* zone);
  virtual void addCheckButton(const char* label, float* zone);
  virtual void addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step);
  virtual void addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step);
  virtual void addNumEntry(const char* label, float* zone, float init, float min, float max, float step);

  virtual void addHorizontalBargraph(const char* label, float* zone, float min, float max);
  virtual void addVerticalBargraph(const char* label, float* zone, float min, float max);
    
  virtual void addSoundfile(const char* label, const char* filename, Soundfile** sf_zone) {}
  
  virtual void openTabBox(const char* label);
  virtual void openHorizontalBox(const char* label);
  virtual void openVerticalBox(const char* label);
  virtual void closeBox();
	
  virtual void run();
};

static std::string mangle(const char *name, int level, const char *s)
{
  const char *s0 = s;
  std::string t = "";
  if (!s) return t;
  // Get rid of bogus "0x00" labels in recent Faust revisions. Also, for
  // backward compatibility with old Faust versions, make sure that default
  // toplevel groups and explicit toplevel groups with an empty label are
  // treated alike (these both return "0x00" labels in the latest Faust, but
  // would be treated inconsistently in earlier versions).
  if (!*s || strcmp(s, "0x00") == 0) {
    if (level == 0)
      // toplevel group with empty label, map to dsp name
      s = name;
    else
      // empty label
      s = "";
  }
  while (*s)
    if (isalnum(*s))
      t += *(s++);
    else {
      const char *s1 = s;
      while (*s && !isalnum(*s)) ++s;
      if (s1 != s0 && *s) t += "-";
    }
  return t;
}

static std::string normpath(std::string path)
{
  path = std::string("/")+path;
  int pos = path.find("//");
  while (pos >= 0) {
    path.erase(pos, 1);
    pos = path.find("//");
  }
  size_t len = path.length();
  if (len > 1 && path[len-1] == '/')
    path.erase(len-1, 1);
  return path;
}

static std::string pathcat(std::string path, std::string label)
{
  if (path.empty())
    return normpath(label);
  else if (label.empty())
    return normpath(path);
  else
    return normpath(path+"/"+label);
}

PdUI::PdUI()
{
  nelems = level = 0;
  elems = NULL;
  name = "";
  path = "";
}

PdUI::PdUI(const char *nm, const char *s)
{
  nelems = level = 0;
  elems = NULL;
  name = nm?nm:"";
  path = s?s:"";
}

PdUI::~PdUI()
{
  if (elems) {
    for (int i = 0; i < nelems; i++)
      if (elems[i].label)
	free(elems[i].label);
    free(elems);
  }
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = NULL;
  elems[nelems].init = 0.0;
  elems[nelems].min = 0.0;
  elems[nelems].max = 0.0;
  elems[nelems].step = 0.0;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = 0.0;
  elems[nelems].min = 0.0;
  elems[nelems].max = 1.0;
  elems[nelems].step = 1.0;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone,
			  float init, float min, float max, float step)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = init;
  elems[nelems].min = min;
  elems[nelems].max = max;
  elems[nelems].step = step;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone,
			  float min, float max)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = 0.0;
  elems[nelems].min = min;
  elems[nelems].max = max;
  elems[nelems].step = 0.0;
  nelems++;
}

void PdUI::addButton(const char* label, float* zone)
{ add_elem(UI_BUTTON, label, zone); }
void PdUI::addCheckButton(const char* label, float* zone)
{ add_elem(UI_CHECK_BUTTON, label, zone); }
void PdUI::addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_V_SLIDER, label, zone, init, min, max, step); }
void PdUI::addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_H_SLIDER, label, zone, init, min, max, step); }
void PdUI::addNumEntry(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_NUM_ENTRY, label, zone, init, min, max, step); }

void PdUI::addHorizontalBargraph(const char* label, float* zone, float min, float max)
{ add_elem(UI_H_BARGRAPH, label, zone, min, max); }
void PdUI::addVerticalBargraph(const char* label, float* zone, float min, float max)
{ add_elem(UI_V_BARGRAPH, label, zone, min, max); }

void PdUI::openTabBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::openHorizontalBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::openVerticalBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::closeBox()
{
  int pos = path.rfind("/");
  if (pos < 0) pos = 0;
  path.erase(pos);
  level--;
}

void PdUI::run() {}

/******************************************************************************
*******************************************************************************

			    FAUST DSP

*******************************************************************************
*******************************************************************************/

//----------------------------------------------------------------------------
//  FAUST generated signal processor
//----------------------------------------------------------------------------
		
#ifndef FAUSTFLOAT
#define FAUSTFLOAT float
#endif 

#include <algorithm>
#include <cmath>
#include <cstdint>
#include <math.h>

#ifndef FAUSTCLASS 
#define FAUSTCLASS mydsp
#endif

#ifdef __APPLE__ 
#define exp10f __exp10f
#define exp10 __exp10
#endif

#if defined(_WIN32)
#define RESTRICT __restrict
#else
#define RESTRICT __restrict__
#endif

static float mydsp_faustpower2_f(float value) {
	return value * value;
}

class mydsp : public dsp {
	
 private:
	
	FAUSTFLOAT fEntry0;
	int fSampleRate;
	float fConst0;
	FAUSTFLOAT fEntry1;
	FAUSTFLOAT fVslider0;
	FAUSTFLOAT fEntry2;
	FAUSTFLOAT fEntry3;
	FAUSTFLOAT fVslider1;
	FAUSTFLOAT fEntry4;
	FAUSTFLOAT fEntry5;
	FAUSTFLOAT fVslider2;
	FAUSTFLOAT fEntry6;
	FAUSTFLOAT fEntry7;
	FAUSTFLOAT fVslider3;
	FAUSTFLOAT fEntry8;
	FAUSTFLOAT fEntry9;
	FAUSTFLOAT fVslider4;
	FAUSTFLOAT fEntry10;
	FAUSTFLOAT fEntry11;
	FAUSTFLOAT fVslider5;
	FAUSTFLOAT fEntry12;
	FAUSTFLOAT fEntry13;
	FAUSTFLOAT fVslider6;
	FAUSTFLOAT fEntry14;
	FAUSTFLOAT fEntry15;
	FAUSTFLOAT fVslider7;
	FAUSTFLOAT fEntry16;
	FAUSTFLOAT fEntry17;
	FAUSTFLOAT fVslider8;
	FAUSTFLOAT fEntry18;
	FAUSTFLOAT fEntry19;
	FAUSTFLOAT fVslider9;
	float fRec9[3];
	float fRec8[3];
	float fRec7[3];
	float fRec6[3];
	float fRec5[3];
	float fRec4[3];
	float fRec3[3];
	float fRec2[3];
	float fRec1[3];
	float fRec0[3];
	
 public:
	
	void metadata(Meta* m) { 
		m->declare("author", "Grame");
		m->declare("bandfilter.dsp/author", "Grame");
		m->declare("bandfilter.dsp/copyright", "(c)GRAME 2006");
		m->declare("bandfilter.dsp/license", "BSD");
		m->declare("bandfilter.dsp/name", "bandfilter");
		m->declare("bandfilter.dsp/version", "1.0");
		m->declare("compile_options", "-a puredata.cpp -lang cpp -es 1 -mcd 16 -single -ftz 0");
		m->declare("copyright", "(c)GRAME 2006");
		m->declare("filename", "multibandfilter.dsp");
		m->declare("license", "BSD");
		m->declare("math.lib/author", "GRAME");
		m->declare("math.lib/copyright", "GRAME");
		m->declare("math.lib/deprecated", "This library is deprecated and is not maintained anymore. It will be removed in August 2017.");
		m->declare("math.lib/license", "LGPL with exception");
		m->declare("math.lib/name", "Math Library");
		m->declare("math.lib/version", "1.0");
		m->declare("music.lib/author", "GRAME");
		m->declare("music.lib/copyright", "GRAME");
		m->declare("music.lib/deprecated", "This library is deprecated and is not maintained anymore. It will be removed in August 2017.");
		m->declare("music.lib/license", "LGPL with exception");
		m->declare("music.lib/name", "Music Library");
		m->declare("music.lib/version", "1.0");
		m->declare("name", "multibandfilter");
		m->declare("version", "1.0");
	}

	virtual int getNumInputs() {
		return 1;
	}
	virtual int getNumOutputs() {
		return 1;
	}
	
	static void classInit(int sample_rate) {
	}
	
	virtual void instanceConstants(int sample_rate) {
		fSampleRate = sample_rate;
		fConst0 = 3.1415927f / std::min<float>(1.92e+05f, std::max<float>(1.0f, float(fSampleRate)));
	}
	
	virtual void instanceResetUserInterface() {
		fEntry0 = FAUSTFLOAT(1e+04f);
		fEntry1 = FAUSTFLOAT(5e+01f);
		fVslider0 = FAUSTFLOAT(0.0f);
		fEntry2 = FAUSTFLOAT(9e+03f);
		fEntry3 = FAUSTFLOAT(5e+01f);
		fVslider1 = FAUSTFLOAT(0.0f);
		fEntry4 = FAUSTFLOAT(8e+03f);
		fEntry5 = FAUSTFLOAT(5e+01f);
		fVslider2 = FAUSTFLOAT(0.0f);
		fEntry6 = FAUSTFLOAT(7e+03f);
		fEntry7 = FAUSTFLOAT(5e+01f);
		fVslider3 = FAUSTFLOAT(0.0f);
		fEntry8 = FAUSTFLOAT(6e+03f);
		fEntry9 = FAUSTFLOAT(5e+01f);
		fVslider4 = FAUSTFLOAT(0.0f);
		fEntry10 = FAUSTFLOAT(5e+03f);
		fEntry11 = FAUSTFLOAT(5e+01f);
		fVslider5 = FAUSTFLOAT(0.0f);
		fEntry12 = FAUSTFLOAT(4e+03f);
		fEntry13 = FAUSTFLOAT(5e+01f);
		fVslider6 = FAUSTFLOAT(0.0f);
		fEntry14 = FAUSTFLOAT(3e+03f);
		fEntry15 = FAUSTFLOAT(5e+01f);
		fVslider7 = FAUSTFLOAT(0.0f);
		fEntry16 = FAUSTFLOAT(2e+03f);
		fEntry17 = FAUSTFLOAT(5e+01f);
		fVslider8 = FAUSTFLOAT(0.0f);
		fEntry18 = FAUSTFLOAT(1e+03f);
		fEntry19 = FAUSTFLOAT(5e+01f);
		fVslider9 = FAUSTFLOAT(0.0f);
	}
	
	virtual void instanceClear() {
		for (int l0 = 0; l0 < 3; l0 = l0 + 1) {
			fRec9[l0] = 0.0f;
		}
		for (int l1 = 0; l1 < 3; l1 = l1 + 1) {
			fRec8[l1] = 0.0f;
		}
		for (int l2 = 0; l2 < 3; l2 = l2 + 1) {
			fRec7[l2] = 0.0f;
		}
		for (int l3 = 0; l3 < 3; l3 = l3 + 1) {
			fRec6[l3] = 0.0f;
		}
		for (int l4 = 0; l4 < 3; l4 = l4 + 1) {
			fRec5[l4] = 0.0f;
		}
		for (int l5 = 0; l5 < 3; l5 = l5 + 1) {
			fRec4[l5] = 0.0f;
		}
		for (int l6 = 0; l6 < 3; l6 = l6 + 1) {
			fRec3[l6] = 0.0f;
		}
		for (int l7 = 0; l7 < 3; l7 = l7 + 1) {
			fRec2[l7] = 0.0f;
		}
		for (int l8 = 0; l8 < 3; l8 = l8 + 1) {
			fRec1[l8] = 0.0f;
		}
		for (int l9 = 0; l9 < 3; l9 = l9 + 1) {
			fRec0[l9] = 0.0f;
		}
	}
	
	virtual void init(int sample_rate) {
		classInit(sample_rate);
		instanceInit(sample_rate);
	}
	virtual void instanceInit(int sample_rate) {
		instanceConstants(sample_rate);
		instanceResetUserInterface();
		instanceClear();
	}
	
	virtual mydsp* clone() {
		return new mydsp();
	}
	
	virtual int getSampleRate() {
		return fSampleRate;
	}
	
	virtual void buildUserInterface(UI* ui_interface) {
		ui_interface->openHorizontalBox("Multi Band Filter");
		ui_interface->openVerticalBox("peak 0");
		ui_interface->declare(&fEntry19, "style", "knob");
		ui_interface->addNumEntry("Q factor", &fEntry19, FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f), FAUSTFLOAT(1e+02f), FAUSTFLOAT(0.1f));
		ui_interface->declare(&fEntry18, "style", "knob");
		ui_interface->declare(&fEntry18, "unit", "Hz");
		ui_interface->addNumEntry("freq", &fEntry18, FAUSTFLOAT(1e+03f), FAUSTFLOAT(2e+01f), FAUSTFLOAT(2e+04f), FAUSTFLOAT(1.0f));
		ui_interface->declare(&fVslider9, "unit", "dB");
		ui_interface->addVerticalSlider("gain", &fVslider9, FAUSTFLOAT(0.0f), FAUSTFLOAT(-5e+01f), FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openVerticalBox("peak 1");
		ui_interface->declare(&fEntry17, "style", "knob");
		ui_interface->addNumEntry("Q factor", &fEntry17, FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f), FAUSTFLOAT(1e+02f), FAUSTFLOAT(0.1f));
		ui_interface->declare(&fEntry16, "style", "knob");
		ui_interface->declare(&fEntry16, "unit", "Hz");
		ui_interface->addNumEntry("freq", &fEntry16, FAUSTFLOAT(2e+03f), FAUSTFLOAT(2e+01f), FAUSTFLOAT(2e+04f), FAUSTFLOAT(1.0f));
		ui_interface->declare(&fVslider8, "unit", "dB");
		ui_interface->addVerticalSlider("gain", &fVslider8, FAUSTFLOAT(0.0f), FAUSTFLOAT(-5e+01f), FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openVerticalBox("peak 2");
		ui_interface->declare(&fEntry15, "style", "knob");
		ui_interface->addNumEntry("Q factor", &fEntry15, FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f), FAUSTFLOAT(1e+02f), FAUSTFLOAT(0.1f));
		ui_interface->declare(&fEntry14, "style", "knob");
		ui_interface->declare(&fEntry14, "unit", "Hz");
		ui_interface->addNumEntry("freq", &fEntry14, FAUSTFLOAT(3e+03f), FAUSTFLOAT(2e+01f), FAUSTFLOAT(2e+04f), FAUSTFLOAT(1.0f));
		ui_interface->declare(&fVslider7, "unit", "dB");
		ui_interface->addVerticalSlider("gain", &fVslider7, FAUSTFLOAT(0.0f), FAUSTFLOAT(-5e+01f), FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openVerticalBox("peak 3");
		ui_interface->declare(&fEntry13, "style", "knob");
		ui_interface->addNumEntry("Q factor", &fEntry13, FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f), FAUSTFLOAT(1e+02f), FAUSTFLOAT(0.1f));
		ui_interface->declare(&fEntry12, "style", "knob");
		ui_interface->declare(&fEntry12, "unit", "Hz");
		ui_interface->addNumEntry("freq", &fEntry12, FAUSTFLOAT(4e+03f), FAUSTFLOAT(2e+01f), FAUSTFLOAT(2e+04f), FAUSTFLOAT(1.0f));
		ui_interface->declare(&fVslider6, "unit", "dB");
		ui_interface->addVerticalSlider("gain", &fVslider6, FAUSTFLOAT(0.0f), FAUSTFLOAT(-5e+01f), FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openVerticalBox("peak 4");
		ui_interface->declare(&fEntry11, "style", "knob");
		ui_interface->addNumEntry("Q factor", &fEntry11, FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f), FAUSTFLOAT(1e+02f), FAUSTFLOAT(0.1f));
		ui_interface->declare(&fEntry10, "style", "knob");
		ui_interface->declare(&fEntry10, "unit", "Hz");
		ui_interface->addNumEntry("freq", &fEntry10, FAUSTFLOAT(5e+03f), FAUSTFLOAT(2e+01f), FAUSTFLOAT(2e+04f), FAUSTFLOAT(1.0f));
		ui_interface->declare(&fVslider5, "unit", "dB");
		ui_interface->addVerticalSlider("gain", &fVslider5, FAUSTFLOAT(0.0f), FAUSTFLOAT(-5e+01f), FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openVerticalBox("peak 5");
		ui_interface->declare(&fEntry9, "style", "knob");
		ui_interface->addNumEntry("Q factor", &fEntry9, FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f), FAUSTFLOAT(1e+02f), FAUSTFLOAT(0.1f));
		ui_interface->declare(&fEntry8, "style", "knob");
		ui_interface->declare(&fEntry8, "unit", "Hz");
		ui_interface->addNumEntry("freq", &fEntry8, FAUSTFLOAT(6e+03f), FAUSTFLOAT(2e+01f), FAUSTFLOAT(2e+04f), FAUSTFLOAT(1.0f));
		ui_interface->declare(&fVslider4, "unit", "dB");
		ui_interface->addVerticalSlider("gain", &fVslider4, FAUSTFLOAT(0.0f), FAUSTFLOAT(-5e+01f), FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openVerticalBox("peak 6");
		ui_interface->declare(&fEntry7, "style", "knob");
		ui_interface->addNumEntry("Q factor", &fEntry7, FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f), FAUSTFLOAT(1e+02f), FAUSTFLOAT(0.1f));
		ui_interface->declare(&fEntry6, "style", "knob");
		ui_interface->declare(&fEntry6, "unit", "Hz");
		ui_interface->addNumEntry("freq", &fEntry6, FAUSTFLOAT(7e+03f), FAUSTFLOAT(2e+01f), FAUSTFLOAT(2e+04f), FAUSTFLOAT(1.0f));
		ui_interface->declare(&fVslider3, "unit", "dB");
		ui_interface->addVerticalSlider("gain", &fVslider3, FAUSTFLOAT(0.0f), FAUSTFLOAT(-5e+01f), FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openVerticalBox("peak 7");
		ui_interface->declare(&fEntry5, "style", "knob");
		ui_interface->addNumEntry("Q factor", &fEntry5, FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f), FAUSTFLOAT(1e+02f), FAUSTFLOAT(0.1f));
		ui_interface->declare(&fEntry4, "style", "knob");
		ui_interface->declare(&fEntry4, "unit", "Hz");
		ui_interface->addNumEntry("freq", &fEntry4, FAUSTFLOAT(8e+03f), FAUSTFLOAT(2e+01f), FAUSTFLOAT(2e+04f), FAUSTFLOAT(1.0f));
		ui_interface->declare(&fVslider2, "unit", "dB");
		ui_interface->addVerticalSlider("gain", &fVslider2, FAUSTFLOAT(0.0f), FAUSTFLOAT(-5e+01f), FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openVerticalBox("peak 8");
		ui_interface->declare(&fEntry3, "style", "knob");
		ui_interface->addNumEntry("Q factor", &fEntry3, FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f), FAUSTFLOAT(1e+02f), FAUSTFLOAT(0.1f));
		ui_interface->declare(&fEntry2, "style", "knob");
		ui_interface->declare(&fEntry2, "unit", "Hz");
		ui_interface->addNumEntry("freq", &fEntry2, FAUSTFLOAT(9e+03f), FAUSTFLOAT(2e+01f), FAUSTFLOAT(2e+04f), FAUSTFLOAT(1.0f));
		ui_interface->declare(&fVslider1, "unit", "dB");
		ui_interface->addVerticalSlider("gain", &fVslider1, FAUSTFLOAT(0.0f), FAUSTFLOAT(-5e+01f), FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->openVerticalBox("peak 9");
		ui_interface->declare(&fEntry1, "style", "knob");
		ui_interface->addNumEntry("Q factor", &fEntry1, FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f), FAUSTFLOAT(1e+02f), FAUSTFLOAT(0.1f));
		ui_interface->declare(&fEntry0, "style", "knob");
		ui_interface->declare(&fEntry0, "unit", "Hz");
		ui_interface->addNumEntry("freq", &fEntry0, FAUSTFLOAT(1e+04f), FAUSTFLOAT(2e+01f), FAUSTFLOAT(2e+04f), FAUSTFLOAT(1.0f));
		ui_interface->declare(&fVslider0, "unit", "dB");
		ui_interface->addVerticalSlider("gain", &fVslider0, FAUSTFLOAT(0.0f), FAUSTFLOAT(-5e+01f), FAUSTFLOAT(5e+01f), FAUSTFLOAT(0.1f));
		ui_interface->closeBox();
		ui_interface->closeBox();
	}
	
	virtual void compute(int count, FAUSTFLOAT** RESTRICT inputs, FAUSTFLOAT** RESTRICT outputs) {
		FAUSTFLOAT* input0 = inputs[0];
		FAUSTFLOAT* output0 = outputs[0];
		float fSlow0 = std::tan(fConst0 * float(fEntry0));
		float fSlow1 = 2.0f * (mydsp_faustpower2_f(fSlow0) + -1.0f);
		float fSlow2 = float(fEntry1);
		float fSlow3 = std::pow(1e+01f, 0.0f - 0.05f * float(fVslider0)) / fSlow2;
		float fSlow4 = fSlow0 * (fSlow0 - fSlow3) + 1.0f;
		float fSlow5 = 1.0f / (fSlow0 * (fSlow0 + fSlow3) + 1.0f);
		float fSlow6 = std::tan(fConst0 * float(fEntry2));
		float fSlow7 = 2.0f * (mydsp_faustpower2_f(fSlow6) + -1.0f);
		float fSlow8 = float(fEntry3);
		float fSlow9 = std::pow(1e+01f, 0.0f - 0.05f * float(fVslider1)) / fSlow8;
		float fSlow10 = fSlow6 * (fSlow6 - fSlow9) + 1.0f;
		float fSlow11 = 1.0f / (fSlow6 * (fSlow6 + fSlow9) + 1.0f);
		float fSlow12 = std::tan(fConst0 * float(fEntry4));
		float fSlow13 = 2.0f * (mydsp_faustpower2_f(fSlow12) + -1.0f);
		float fSlow14 = float(fEntry5);
		float fSlow15 = std::pow(1e+01f, 0.0f - 0.05f * float(fVslider2)) / fSlow14;
		float fSlow16 = fSlow12 * (fSlow12 - fSlow15) + 1.0f;
		float fSlow17 = 1.0f / (fSlow12 * (fSlow12 + fSlow15) + 1.0f);
		float fSlow18 = std::tan(fConst0 * float(fEntry6));
		float fSlow19 = 2.0f * (mydsp_faustpower2_f(fSlow18) + -1.0f);
		float fSlow20 = float(fEntry7);
		float fSlow21 = std::pow(1e+01f, 0.0f - 0.05f * float(fVslider3)) / fSlow20;
		float fSlow22 = fSlow18 * (fSlow18 - fSlow21) + 1.0f;
		float fSlow23 = 1.0f / (fSlow18 * (fSlow18 + fSlow21) + 1.0f);
		float fSlow24 = std::tan(fConst0 * float(fEntry8));
		float fSlow25 = 2.0f * (mydsp_faustpower2_f(fSlow24) + -1.0f);
		float fSlow26 = float(fEntry9);
		float fSlow27 = std::pow(1e+01f, 0.0f - 0.05f * float(fVslider4)) / fSlow26;
		float fSlow28 = fSlow24 * (fSlow24 - fSlow27) + 1.0f;
		float fSlow29 = 1.0f / (fSlow24 * (fSlow24 + fSlow27) + 1.0f);
		float fSlow30 = std::tan(fConst0 * float(fEntry10));
		float fSlow31 = 2.0f * (mydsp_faustpower2_f(fSlow30) + -1.0f);
		float fSlow32 = float(fEntry11);
		float fSlow33 = std::pow(1e+01f, 0.0f - 0.05f * float(fVslider5)) / fSlow32;
		float fSlow34 = fSlow30 * (fSlow30 - fSlow33) + 1.0f;
		float fSlow35 = 1.0f / (fSlow30 * (fSlow30 + fSlow33) + 1.0f);
		float fSlow36 = std::tan(fConst0 * float(fEntry12));
		float fSlow37 = 2.0f * (mydsp_faustpower2_f(fSlow36) + -1.0f);
		float fSlow38 = float(fEntry13);
		float fSlow39 = std::pow(1e+01f, 0.0f - 0.05f * float(fVslider6)) / fSlow38;
		float fSlow40 = fSlow36 * (fSlow36 - fSlow39) + 1.0f;
		float fSlow41 = 1.0f / (fSlow36 * (fSlow36 + fSlow39) + 1.0f);
		float fSlow42 = std::tan(fConst0 * float(fEntry14));
		float fSlow43 = 2.0f * (mydsp_faustpower2_f(fSlow42) + -1.0f);
		float fSlow44 = float(fEntry15);
		float fSlow45 = std::pow(1e+01f, 0.0f - 0.05f * float(fVslider7)) / fSlow44;
		float fSlow46 = fSlow42 * (fSlow42 - fSlow45) + 1.0f;
		float fSlow47 = 1.0f / (fSlow42 * (fSlow42 + fSlow45) + 1.0f);
		float fSlow48 = std::tan(fConst0 * float(fEntry16));
		float fSlow49 = 2.0f * (mydsp_faustpower2_f(fSlow48) + -1.0f);
		float fSlow50 = float(fEntry17);
		float fSlow51 = std::pow(1e+01f, 0.0f - 0.05f * float(fVslider8)) / fSlow50;
		float fSlow52 = fSlow48 * (fSlow48 - fSlow51) + 1.0f;
		float fSlow53 = 1.0f / (fSlow48 * (fSlow48 + fSlow51) + 1.0f);
		float fSlow54 = std::tan(fConst0 * float(fEntry18));
		float fSlow55 = 2.0f * (mydsp_faustpower2_f(fSlow54) + -1.0f);
		float fSlow56 = float(fEntry19);
		float fSlow57 = std::pow(1e+01f, 0.0f - 0.05f * float(fVslider9)) / fSlow56;
		float fSlow58 = fSlow54 * (fSlow54 - fSlow57) + 1.0f;
		float fSlow59 = 1.0f / (fSlow54 * (fSlow54 + fSlow57) + 1.0f);
		float fSlow60 = 1.0f / fSlow56;
		float fSlow61 = 1.0f - fSlow54 * (fSlow60 - fSlow54);
		float fSlow62 = fSlow54 * (fSlow54 + fSlow60) + 1.0f;
		float fSlow63 = 1.0f / fSlow50;
		float fSlow64 = 1.0f - fSlow48 * (fSlow63 - fSlow48);
		float fSlow65 = fSlow48 * (fSlow48 + fSlow63) + 1.0f;
		float fSlow66 = 1.0f / fSlow44;
		float fSlow67 = 1.0f - fSlow42 * (fSlow66 - fSlow42);
		float fSlow68 = fSlow42 * (fSlow42 + fSlow66) + 1.0f;
		float fSlow69 = 1.0f / fSlow38;
		float fSlow70 = 1.0f - fSlow36 * (fSlow69 - fSlow36);
		float fSlow71 = fSlow36 * (fSlow36 + fSlow69) + 1.0f;
		float fSlow72 = 1.0f / fSlow32;
		float fSlow73 = 1.0f - fSlow30 * (fSlow72 - fSlow30);
		float fSlow74 = fSlow30 * (fSlow30 + fSlow72) + 1.0f;
		float fSlow75 = 1.0f / fSlow26;
		float fSlow76 = 1.0f - fSlow24 * (fSlow75 - fSlow24);
		float fSlow77 = fSlow24 * (fSlow24 + fSlow75) + 1.0f;
		float fSlow78 = 1.0f / fSlow20;
		float fSlow79 = 1.0f - fSlow18 * (fSlow78 - fSlow18);
		float fSlow80 = fSlow18 * (fSlow18 + fSlow78) + 1.0f;
		float fSlow81 = 1.0f / fSlow14;
		float fSlow82 = 1.0f - fSlow12 * (fSlow81 - fSlow12);
		float fSlow83 = fSlow12 * (fSlow12 + fSlow81) + 1.0f;
		float fSlow84 = 1.0f / fSlow8;
		float fSlow85 = 1.0f - fSlow6 * (fSlow84 - fSlow6);
		float fSlow86 = fSlow6 * (fSlow6 + fSlow84) + 1.0f;
		float fSlow87 = 1.0f / fSlow2;
		float fSlow88 = 1.0f - fSlow0 * (fSlow87 - fSlow0);
		float fSlow89 = fSlow0 * (fSlow0 + fSlow87) + 1.0f;
		for (int i0 = 0; i0 < count; i0 = i0 + 1) {
			float fTemp0 = fSlow1 * fRec0[1];
			float fTemp1 = fSlow7 * fRec1[1];
			float fTemp2 = fSlow13 * fRec2[1];
			float fTemp3 = fSlow19 * fRec3[1];
			float fTemp4 = fSlow25 * fRec4[1];
			float fTemp5 = fSlow31 * fRec5[1];
			float fTemp6 = fSlow37 * fRec6[1];
			float fTemp7 = fSlow43 * fRec7[1];
			float fTemp8 = fSlow49 * fRec8[1];
			float fTemp9 = fSlow55 * fRec9[1];
			fRec9[0] = float(input0[i0]) - fSlow59 * (fSlow58 * fRec9[2] + fTemp9);
			fRec8[0] = fSlow59 * (fTemp9 + fSlow62 * fRec9[0] + fSlow61 * fRec9[2]) - fSlow53 * (fSlow52 * fRec8[2] + fTemp8);
			fRec7[0] = fSlow53 * (fTemp8 + fSlow65 * fRec8[0] + fSlow64 * fRec8[2]) - fSlow47 * (fSlow46 * fRec7[2] + fTemp7);
			fRec6[0] = fSlow47 * (fTemp7 + fSlow68 * fRec7[0] + fSlow67 * fRec7[2]) - fSlow41 * (fSlow40 * fRec6[2] + fTemp6);
			fRec5[0] = fSlow41 * (fTemp6 + fSlow71 * fRec6[0] + fSlow70 * fRec6[2]) - fSlow35 * (fSlow34 * fRec5[2] + fTemp5);
			fRec4[0] = fSlow35 * (fTemp5 + fSlow74 * fRec5[0] + fSlow73 * fRec5[2]) - fSlow29 * (fSlow28 * fRec4[2] + fTemp4);
			fRec3[0] = fSlow29 * (fTemp4 + fSlow77 * fRec4[0] + fSlow76 * fRec4[2]) - fSlow23 * (fSlow22 * fRec3[2] + fTemp3);
			fRec2[0] = fSlow23 * (fTemp3 + fSlow80 * fRec3[0] + fSlow79 * fRec3[2]) - fSlow17 * (fSlow16 * fRec2[2] + fTemp2);
			fRec1[0] = fSlow17 * (fTemp2 + fSlow83 * fRec2[0] + fSlow82 * fRec2[2]) - fSlow11 * (fSlow10 * fRec1[2] + fTemp1);
			fRec0[0] = fSlow11 * (fTemp1 + fSlow86 * fRec1[0] + fSlow85 * fRec1[2]) - fSlow5 * (fSlow4 * fRec0[2] + fTemp0);
			output0[i0] = FAUSTFLOAT(fSlow5 * (fTemp0 + fSlow89 * fRec0[0] + fSlow88 * fRec0[2]));
			fRec9[2] = fRec9[1];
			fRec9[1] = fRec9[0];
			fRec8[2] = fRec8[1];
			fRec8[1] = fRec8[0];
			fRec7[2] = fRec7[1];
			fRec7[1] = fRec7[0];
			fRec6[2] = fRec6[1];
			fRec6[1] = fRec6[0];
			fRec5[2] = fRec5[1];
			fRec5[1] = fRec5[0];
			fRec4[2] = fRec4[1];
			fRec4[1] = fRec4[0];
			fRec3[2] = fRec3[1];
			fRec3[1] = fRec3[0];
			fRec2[2] = fRec2[1];
			fRec2[1] = fRec2[0];
			fRec1[2] = fRec1[1];
			fRec1[1] = fRec1[0];
			fRec0[2] = fRec0[1];
			fRec0[1] = fRec0[0];
		}
	}

};

#include <stdio.h>
#include <string>
#include "m_pd.h"

#define sym(name) xsym(name)
#define xsym(name) #name
#define faust_setup(name) xfaust_setup(name)
#define xfaust_setup(name) name ## _tilde_setup(void)

// time for "active" toggle xfades in secs
#define XFADE_TIME 0.1f

static t_class *faust_class;

struct t_faust {
  t_object x_obj;
#ifdef __MINGW32__
  /* This seems to be necessary as some as yet undetermined Pd routine seems
     to write past the end of x_obj on Windows. */
  int fence; /* dummy field (not used) */
#endif
  mydsp *dsp;
  PdUI *ui;
  std::string *label;
  int active, xfade, n_xfade, rate, n_in, n_out;
  t_sample **inputs, **outputs, **buf;
  t_outlet *out;
  t_sample f;
};

static t_symbol *s_button, *s_checkbox, *s_vslider, *s_hslider, *s_nentry,
  *s_vbargraph, *s_hbargraph;

static inline void zero_samples(int k, int n, t_sample **out)
{
  for (int i = 0; i < k; i++)
#ifdef __STDC_IEC_559__
    /* IEC 559 a.k.a. IEEE 754 floats can be initialized faster like this */
    memset(out[i], 0, n*sizeof(t_sample));
#else
    for (int j = 0; j < n; j++)
      out[i][j] = 0.0f;
#endif
}

static inline void copy_samples(int k, int n, t_sample **out, t_sample **in)
{
  for (int i = 0; i < k; i++)
    memcpy(out[i], in[i], n*sizeof(t_sample));
}

static t_int *faust_perform(t_int *w)
{
  t_faust *x = (t_faust *)(w[1]);
  int n = (int)(w[2]);
  if (!x->dsp || !x->buf) return (w+3);
  AVOIDDENORMALS;
  if (x->xfade > 0) {
    float d = 1.0f/x->n_xfade, f = (x->xfade--)*d;
    d = d/n;
    x->dsp->compute(n, x->inputs, x->buf);
    if (x->active)
      if (x->n_in == x->n_out)
	/* xfade inputs -> buf */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->inputs[i][j]+(1.0f-f)*x->buf[i][j];
      else
	/* xfade 0 -> buf */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = (1.0f-f)*x->buf[i][j];
    else
      if (x->n_in == x->n_out)
	/* xfade buf -> inputs */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->buf[i][j]+(1.0f-f)*x->inputs[i][j];
      else
	/* xfade buf -> 0 */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->buf[i][j];
  } else if (x->active) {
    x->dsp->compute(n, x->inputs, x->buf);
    copy_samples(x->n_out, n, x->outputs, x->buf);
  } else if (x->n_in == x->n_out) {
    copy_samples(x->n_out, n, x->buf, x->inputs);
    copy_samples(x->n_out, n, x->outputs, x->buf);
  } else
    zero_samples(x->n_out, n, x->outputs);
  return (w+3);
}

static void faust_dsp(t_faust *x, t_signal **sp)
{
  int n = sp[0]->s_n, sr = (int)sp[0]->s_sr;
  if (x->rate <= 0) {
    /* default sample rate is whatever Pd tells us */
    PdUI *ui = x->ui;
    float *z = NULL;
    if (ui->nelems > 0 &&
	(z = (float*)malloc(ui->nelems*sizeof(float)))) {
      /* save the current control values */
      for (int i = 0; i < ui->nelems; i++)
	if (ui->elems[i].zone)
	  z[i] = *ui->elems[i].zone;
    }
    /* set the proper sample rate; this requires reinitializing the dsp */
    x->rate = sr;
    x->dsp->init(sr);
    if (z) {
      /* restore previous control values */
      for (int i = 0; i < ui->nelems; i++)
	if (ui->elems[i].zone)
	  *ui->elems[i].zone = z[i];
      free(z);
    }
  }
  if (n > 0)
    x->n_xfade = (int)(x->rate*XFADE_TIME/n);
  dsp_add(faust_perform, 2, x, n);
  for (int i = 0; i < x->n_in; i++)
    x->inputs[i] = sp[i+1]->s_vec;
  for (int i = 0; i < x->n_out; i++)
    x->outputs[i] = sp[x->n_in+i+1]->s_vec;
  if (x->buf != NULL)
    for (int i = 0; i < x->n_out; i++) {
      x->buf[i] = (t_sample*)malloc(n*sizeof(t_sample));
      if (x->buf[i] == NULL) {
	for (int j = 0; j < i; j++)
	  free(x->buf[j]);
	free(x->buf);
	x->buf = NULL;
	break;
      }
    }
}

static int pathcmp(const char *s, const char *t)
{
  int n = strlen(s), m = strlen(t);
  if (n == 0 || m == 0)
    return 0;
  else if (t[0] == '/')
    return strcmp(s, t);
  else if (n <= m || s[n-m-1] != '/')
    return strcmp(s+1, t);
  else
    return strcmp(s+n-m, t);
}

static void faust_any(t_faust *x, t_symbol *s, int argc, t_atom *argv)
{
  if (!x->dsp) return;
  PdUI *ui = x->ui;
  if (s == &s_bang) {
    for (int i = 0; i < ui->nelems; i++)
      if (ui->elems[i].label && ui->elems[i].zone) {
	t_atom args[6];
	t_symbol *_s;
	switch (ui->elems[i].type) {
	case UI_BUTTON:
	  _s = s_button;
	  break;
	case UI_CHECK_BUTTON:
	  _s = s_checkbox;
	  break;
	case UI_V_SLIDER:
	  _s = s_vslider;
	  break;
	case UI_H_SLIDER:
	  _s = s_hslider;
	  break;
	case UI_NUM_ENTRY:
	  _s = s_nentry;
	  break;
	case UI_V_BARGRAPH:
	  _s = s_vbargraph;
	  break;
	case UI_H_BARGRAPH:
	  _s = s_hbargraph;
	  break;
	default:
	  continue;
	}
	SETSYMBOL(&args[0], gensym(ui->elems[i].label));
	SETFLOAT(&args[1], *ui->elems[i].zone);
	SETFLOAT(&args[2], ui->elems[i].init);
	SETFLOAT(&args[3], ui->elems[i].min);
	SETFLOAT(&args[4], ui->elems[i].max);
	SETFLOAT(&args[5], ui->elems[i].step);
	outlet_anything(x->out, _s, 6, args);
      }
  } else {
    const char *label = s->s_name;
    int count = 0;
    for (int i = 0; i < ui->nelems; i++)
      if (ui->elems[i].label &&
	  pathcmp(ui->elems[i].label, label) == 0) {
	if (argc == 0) {
	  if (ui->elems[i].zone) {
	    t_atom arg;
	    SETFLOAT(&arg, *ui->elems[i].zone);
	    outlet_anything(x->out, gensym(ui->elems[i].label), 1, &arg);
	  }
	  ++count;
	} else if (argc == 1 &&
		   (argv[0].a_type == A_FLOAT ||
		    argv[0].a_type == A_DEFFLOAT) &&
		   ui->elems[i].zone) {
	  float f = atom_getfloat(argv);
	  *ui->elems[i].zone = f;
	  ++count;
	} else
	  pd_error(x, "[faust] %s: bad control argument: %s",
		   x->label->c_str(), label);
      }
    if (count == 0 && strcmp(label, "active") == 0) {
      if (argc == 0) {
	t_atom arg;
	SETFLOAT(&arg, (float)x->active);
	outlet_anything(x->out, gensym((char*)"active"), 1, &arg);
      } else if (argc == 1 &&
		 (argv[0].a_type == A_FLOAT ||
		  argv[0].a_type == A_DEFFLOAT)) {
	float f = atom_getfloat(argv);
	x->active = (int)f;
	x->xfade = x->n_xfade;
      }
    }
  }
}

static void faust_free(t_faust *x)
{
  if (x->label) delete x->label;
  if (x->dsp) delete x->dsp;
  if (x->ui) delete x->ui;
  if (x->inputs) free(x->inputs);
  if (x->outputs) free(x->outputs);
  if (x->buf) {
    for (int i = 0; i < x->n_out; i++)
      if (x->buf[i]) free(x->buf[i]);
    free(x->buf);
  }
}

static void *faust_new(t_symbol *s, int argc, t_atom *argv)
{
  t_faust *x = (t_faust*)pd_new(faust_class);
  int sr = -1;
  t_symbol *id = NULL;
  x->active = 1;
  for (int i = 0; i < argc; i++)
    if (argv[i].a_type == A_FLOAT || argv[i].a_type == A_DEFFLOAT)
      sr = (int)argv[i].a_w.w_float;
    else if (argv[i].a_type == A_SYMBOL || argv[i].a_type == A_DEFSYMBOL)
      id = argv[i].a_w.w_symbol;
  x->rate = sr;
  if (sr <= 0) sr = 44100;
  x->xfade = 0; x->n_xfade = (int)(sr*XFADE_TIME/64);
  x->inputs = x->outputs = x->buf = NULL;
    x->label = new std::string(sym(mydsp) "~");
  x->dsp = new mydsp();
  x->ui = new PdUI(sym(mydsp), id?id->s_name:NULL);
  if (!x->dsp || !x->ui || !x->label) goto error;
  if (id) {
    *x->label += " ";
    *x->label += id->s_name;
  }
  x->n_in = x->dsp->getNumInputs();
  x->n_out = x->dsp->getNumOutputs();
  if (x->n_in > 0)
    x->inputs = (t_sample**)malloc(x->n_in*sizeof(t_sample*));
  if (x->n_out > 0) {
    x->outputs = (t_sample**)malloc(x->n_out*sizeof(t_sample*));
    x->buf = (t_sample**)malloc(x->n_out*sizeof(t_sample*));
  }
  if ((x->n_in > 0 && x->inputs == NULL) ||
      (x->n_out > 0 && (x->outputs == NULL || x->buf == NULL)))
    goto error;
  for (int i = 0; i < x->n_out; i++)
    x->buf[i] = NULL;
  x->dsp->init(sr);
  x->dsp->buildUserInterface(x->ui);
  for (int i = 0; i < x->n_in; i++)
    inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
  x->out = outlet_new(&x->x_obj, 0);
  for (int i = 0; i < x->n_out; i++)
    outlet_new(&x->x_obj, &s_signal);
  return (void *)x;
 error:
  faust_free(x);
  x->dsp = NULL; x->ui = NULL;
  x->inputs = x->outputs = x->buf = NULL;
  return (void *)x;
}

extern "C" void faust_setup(mydsp)
{
  t_symbol *s = gensym(sym(mydsp) "~");
  faust_class =
    class_new(s, (t_newmethod)faust_new, (t_method)faust_free,
	      sizeof(t_faust), CLASS_DEFAULT,
	      A_GIMME, A_NULL);
  class_addmethod(faust_class, (t_method)faust_dsp, gensym((char*)"dsp"), A_NULL);
  class_addanything(faust_class, faust_any);
  class_addmethod(faust_class, nullfn, &s_signal, A_NULL);
  s_button = gensym((char*)"button");
  s_checkbox = gensym((char*)"checkbox");
  s_vslider = gensym((char*)"vslider");
  s_hslider = gensym((char*)"hslider");
  s_nentry = gensym((char*)"nentry");
  s_vbargraph = gensym((char*)"vbargraph");
  s_hbargraph = gensym((char*)"hbargraph");
  /* give some indication that we're loaded and ready to go */
  mydsp dsp = mydsp();
  post("[faust] %s: %d inputs, %d outputs", sym(mydsp) "~",
       dsp.getNumInputs(), dsp.getNumOutputs());
}

#endif
